import java.io.*;

/**
* A class to represent a 3D object
* @author Ho Ng
* @version 01/09/1998
*/
public class Object3D
{
    /**
    * Polygon3D[] p <br>
    * purpose : store the polygons of the object
    */
    public Polygon3D[] p;

    /**
    * int count <br>
    * purpose : total number of polygons in the object
    */
    public int count;

    /**
    * Default constructor
    * purpose : initialize an object with no polygons
    */
    public Object3D()
    {
        count = 0;
    }

    /**
    * Constructor
    * purpose : initialize an object with some polygons
    * @param total total number of polygons in the object
    */
    public Object3D(int total)
    {
        p = new Polygon3D[total];
        count = 0;
    }

    /**
    * Constructor
    * purpose : initialize an object by the data obtained from the input reader
    * @param data BufferedReader which contains the data of the Object3D
    */
    public Object3D(BufferedReader data)
    {
        try
        {
            String str = data.readLine();
            Sscanf input = new Sscanf(str);
            count = Integer.valueOf(input.nextWord()).intValue();
            p = new Polygon3D[count];

                for (int i = 0; i < count; i++)
                {
                    str = data.readLine();
                p[i] = new Polygon3D(str);
                }
            }
            catch (Exception e)
            {
                System.out.println("Object3D : incorrect input data " + e);
                count = 0;
            }
    }

    /**
    * Copy constructor
    * purpose : initialize an object to an input Object3D x
    */
    public Object3D(Object3D x)
    {
        count = x.count;
        p = new Polygon3D[count];
        for (int i = 0; i < x.count; i++)
            p[i] = new Polygon3D(x.p[i]);
    }

    private void addPolygon(Polygon3D poly)
    {
        // ...
    }

    /**
    * purpose : multiply each point of the Object3D by a matrix
    * @param m Matrix
    * @param obj Object3D
    * @return a new copy of the transformed Object3D
    */
    public static Object3D transformObjectQ(Matrix m, Object3D obj)
    {
        Object3D result = new Object3D(obj.count);

        for (int i = 0; i < obj.count; i++)
            result.p[i] = Polygon3D.transformPolygonQ(m, obj.p[i]);

        return result;
    }

    /**
    * purpose : multiply each point of the Object3D by a matrix
    * @param m Matrix
    * @param obj Object3D
    * @return a new copy of the transformed Object3D
    */
    public static Object3D transformObject(Matrix m, Object3D obj)
    {
        Object3D result = new Object3D(obj.count);

        for (int i = 0; i < obj.count; i++)
            result.p[i] = Polygon3D.transformPolygon(m, obj.p[i]);

        return result;
    }
};
