import java.io.*;

/**
* A class to represent a 3D polygon
* @author Ho Ng
* @version 01/09/1998
*/
public class Polygon3D
{
    /**
    * Vector3D[] c <br>
    * purpose : store the vertices of the polygon
    */
    public Vector3D[] c;

    /**
    * int count <br>
    * purpose : total number of vertices in the polygon
    */
    public int count;

    /**
    * Default constructor <br>
    * purpose : initialize a polygon with no vertices
    */
    public Polygon3D()
    {
        count = 0;
    }

    /**
    * Constructor <br>
    * purpose : initialize a polygon with some vertices
    * @param total total number of vertices in the polygon
    */
    public Polygon3D(int total)
    {
        c = new Vector3D[total];
        count = total;
    }

    /**
    * Constructor <br>
    * purpose : initialize a polygon by the data in the input String
    * @param str a String contains the input data
    */
    Polygon3D(String str)
    {
        double x, y, z;

        Sscanf input = new Sscanf(str);
        String value = input.nextWord();

        count = Integer.valueOf(value).intValue();
        c = new Vector3D[count];

        for (int i = 0; i < count; i++)
        {
            value = input.nextWord();
            x = Double.valueOf(value).doubleValue();
            value = input.nextWord();
            y = Double.valueOf(value).doubleValue();
            value = input.nextWord();
            z = Double.valueOf(value).doubleValue();
            c[i] = new Vector3D(x, y, z);
        }
    }

    /**
    * Copy constructor <br>
    * purpose : initialize a polygon to an input Polygon3D x
    * @param x input Polygon3D
    */
    Polygon3D(Polygon3D x)
    {
        count = x.count;
        c = new Vector3D[count];
        for (int i = 0; i < count; i++)
            c[i] = new Vector3D(x.c[i]);
    }

    /**
    * purpose : multiply each point of the polygon by a matrix
    * @param m Matrix
    * @param x Polygon3D
    * @return a new copy of the transformed Polygon3D
    */
    public static Polygon3D transformPolygonQ(Matrix m, Polygon3D x)
    {
        Polygon3D result = new Polygon3D(x.count);

        for (int i = 0; i < x.count; i++)
            result.c[i] = Vector3D.transformPointQ(m, x.c[i]);

        return result;
    }

    /**
    * purpose : multiply each point of the polygon by a matrix
    * @param m Matrix
    * @param x Polygon3D
    * @return a new copy of the transformed Polygon3D
    */
    public static Polygon3D transformPolygon(Matrix m, Polygon3D x)
    {
        Polygon3D result = new Polygon3D(x.count);

        for (int i = 0; i < x.count; i++)
            result.c[i] = Vector3D.transformPoint(m, x.c[i]);

        return result;
    }
};
